@@ -33,6 +33,7 @@ Usage
 
   roommate = Dude(feet.options.target)
   feet.set_value('stench', rommmate.gym_socks())
+  feet.set_status_message('Current stench level (%s)' % rommmate.gym_socks())
 
   feet.finish()
 
@@ -44,7 +45,7 @@ API Specification
 
 """
 
-__version__ = '0.1.1'
+__version__ = '0.1.3'
 __all__ = ['Plugin']
 
 # We need the option parser primarily to provide an instruction harness to the
@@ -100,6 +101,7 @@ class Plugin:
         self.__req_option = []     # Options considered required for operation.
         self.__started = False     # Has 'start' been called yet?
         self.__exit_status = None  # 'ok', 'warning', or 'critical'.
+        self.__exit_message = None # User specified status, filtered.
 
         # With our variables out of the way, let's start up the option parser
         # with a version and verbose setting, along with a sane usage
@@ -214,9 +216,9 @@ class Plugin:
 
         .. note::
 
-           If **warning** or **critical** is enabled, failing to provide them
-           to the plugin will result in an automatic exit and presentation of
-           the usage documentation.
+           If **warning** or **critical** is enabled and set as required,
+           failing to provide them to the plugin will result in an automatic
+           exit and presentation of the usage documentation.
 
         When plugins make use of multiple performance metrics, they may also
         have different scales, and hence warning/critical ranges involved.
@@ -224,27 +226,36 @@ class Plugin:
         comma-delimited format.
 
         :param status_type: Should be either `warning` or `critical`.
+        :type status_type: String.
         :param required: Should this threshold be a required option?
         :type required: True or False. Default False.
+
         """
-        assert status_type in ('warning', 'critical'), (
-            "Status_type can only be one of *warning* or *critical*!")
+        try:
+            status_type = status_type.lower()
+            assert status_type in ('warning', 'critical')
+        except (AttributeError, AssertionError):
+            self.unknown_error(
+                "Status_type can only be one of *warning* or *critical*!")
 
         self.add_option(status_type[0], status_type,
             "Set the %s notification level." % status_type,
             required=required, action='callback', callback=convert_range)
 
-
     def finish(self):
         """
-        Checks the proper warning/errors and prints out all results.
+        Prints out all results in a form Nagios understands.
 
-        Once all values have been set through ordinary means, it remains to
-        check them against our warning/critical status ranges if necessary
-        and output all our wonderful results so Nagios can consume them.
+        The process of setting a value through 'set_value' actually applies
+        warning and critical threshold checks, so by now, all is over but the
+        shouting. This method prints out the status and performance data in a
+        manner Nagios will understand.
 
         """
-        assert self.__started, "Start method must be called first!"
+        try:
+            assert self.__started
+        except AssertionError:
+            self.unknown_error("Start method must be called first!")
 
         exit_value = 0
         if self.__exit_status == 'warning':
@@ -271,7 +282,11 @@ class Plugin:
         if len(perfs) > 0:
             perf_string = '|' + ' '.join(["%s" % item for item in perfs])
 
-        print "Status %s%s" % (self.__exit_status.capitalize(), perf_string)
+        exit_status = self.__exit_status.capitalize()
+        if self.__exit_message is not None:
+            exit_status += ', %s' % self.__exit_message
+
+        print 'Status ' + exit_status + perf_string
         sys.exit(exit_value)
 
     def set_value(self, name, val, **kwargs):
@@ -296,7 +311,7 @@ class Plugin:
 
         :param name: Name of the performance setting.
         :param val: Value observed for this iteration.
-        :type val: float or None
+        :type val: float
         :param lowest: Minimum possible value for this setting. Optional.
         :type lowest: float or None
         :param highest: Maximum possible value for this setting. Optional.
@@ -313,10 +328,13 @@ class Plugin:
         :return string: One of 'ok', 'warning' or 'critical', as to how this
             value compared to the enabled and supplied thresholds.
         """
-        assert self.__started, "Start method must be called first!"
-        assert val is not None
-
-        val = float(val)
+        try:
+            assert self.__started
+            val = float(val)
+        except AssertionError:
+            self.unknown_error("Start method must be called first!")
+        except (TypeError, ValueError):
+            self.unknown_error("Performance measures must be numeric!")
 
         val_dict = {'val': val, 'min': None, 'max': None, 'scale': None,
             'threshold': 1, 'state': 'ok'}
@@ -358,6 +376,44 @@ class Plugin:
 
         return self.__perf[name]['state']
 
+    def set_status_message(self, message):
+        """
+        Set a more informational exit string for plugin status.
+
+        By default, NagAconda exits merely with OK, WARNING, or CRITICAL, a
+        raw interpretation as applied to provided warning and critical
+        thresholds. Clearly this isn't sufficent in many cases where plugins
+        know more than merely the metrics being measured. This method is
+        provided as a means to manually add textual information to the plugin's
+        output that won't interfere with Nagios's parsing of that status.
+
+        Since status may change through the process of the plugin's execution
+        several times, or in accordance to the status of several metrics, this
+        is provided separately from the 'finish' method.
+
+        .. note::
+
+            This is not the same as verbose Nagios output that can span
+            multiple lines. This is just appended to the usual OK / WARNING /
+            CRITICAL status reported back to Nagios.
+
+        :param message: Exit status to include in Nagios output. String
+            will be filtered to remove characters that may cause Nagios to
+            ignore or misinterpret plugin output.
+        :type message: string
+
+        """
+
+        # Nagios considers a pipe (|) a split from STATUS MESSAGE and perf
+        # data. If we replace it with a space, that should safely render the
+        # message safe without risking making it unreadable.
+
+        try:
+            assert message is not None
+            self.__exit_message = message.replace('|', ' ')
+        except (AttributeError, AssertionError):
+            self.unknown_error("Status message must be a standard string!")
+
     def start(self):
         """
         Invokes all preparation steps to retrieve host/service status.
@@ -381,6 +437,28 @@ class Plugin:
         self.__started = True
         self.__exit_status = 'OK'
 
+    def unknown_error(self, message):
+        """
+        The Plugin encountered an unexpected error, and must immediately exit.
+
+        The plugin system itself handles all the necessary details when
+        processing thresholds, which will take care of OK, WARNING, and
+        CRITICAL exit status, but what about UNKNOWN? If some kind of
+        unexpected behavior or parameter, or otherwise misbehaving resource
+        confuses our plugin, we should immediately stop and reconsider.
+        Call this to generate Nagios output with an optional message as to
+        why the plugin quit processing.
+
+        This allows 'finish' to continue to function as an expected exit
+        point full of processed performance metrics without unnecessary
+        error handling/formatting logic.
+
+        :param message: Exit status to include in Nagios output.
+        :type message: string
+
+        """
+        print 'Status Unknown: ' + message
+        sys.exit(3)
 
 def convert_range(option, opt_str, value, parser):
     """
